/**
  * ColorInstanced.fx
  *
  * Author:         Peter B. Tummillo
  * Last Modified:  2/27/12
  * 
  * Description:    Renders an instanced object with a color.
  */

cbuffer cbPerFrame
{
	float4x4 gWVP; 
};


struct VS_IN
{
	float3 posL                  : POSITION;
	float4 color                 : COLOR;
	row_major float4x4 transform : TRANSFORM;
};

struct VS_OUT
{
	float4 posH   : SV_POSITION;
	float4 color  : COLOR;
};


VS_OUT VS(VS_IN input)
{
	VS_OUT output;

	// Determine position of this instance
	float4 instancePos = mul( float4(input.posL, 1.0f), input.transform );

	// Transform vertex position to homogeneous clip space
	output.posH = mul( instancePos, gWVP );
	
	// Just pass color into the pixel shader
	output.color = input.color;

	return output;
}

float4 PS(VS_OUT input) : SV_Target
{
	// Return default color
	return input.color;
}


technique10 ColorInstancedTech
{
	pass P0
	{
		SetVertexShader( CompileShader( vs_4_0, VS() ) );
        SetGeometryShader( NULL );
        SetPixelShader( CompileShader( ps_4_0, PS() ) );
	}
}
