/**
  * Lines.fx
  *
  * Author:        Peter B. Tummillo
  * Last Modified: 1/21/12
  *
  * Description:   Uses a geometry shader to render lines with a specified thickness.
  */

cbuffer cbPerFrame
{
	float4x4 gWVP; 
};


float gLineThickness;


struct Line2D
{
	float3 posL  : POSITION;
	float4 color : COLOR;
};

struct GS_OUT
{
	float4 posH  : SV_POSITION;
	float4 color : COLOR;
};


Line2D VS(Line2D input)
{
	// The geometry shader does all the work
	return input;
}

[maxvertexcount(4)]
void GS(line Line2D input[2], inout TriangleStream<GS_OUT> triStream)
{
	// Color stays the same
	float4 color      = input[0].color;
	
	// Figure out what direction the line is going in
	float3 dir = input[1].posL - input[0].posL;
	
	// Widden the lines by turning them into quads
	// (assume they are always going to be horizontal or vertical)
	float  r = gLineThickness / 2.0f;
	float4 v[4];
	if (dir.x == 0.0f) // vertical
	{
		v[0] = float4(input[0].posL.x - r, input[0].posL.y, 0.0f, 1.0f);
		v[1] = float4(input[0].posL.x + r, input[0].posL.y, 0.0f, 1.0f);
		v[2] = float4(input[1].posL.x - r, input[1].posL.y, 0.0f, 1.0f);
		v[3] = float4(input[1].posL.x + r, input[1].posL.y, 0.0f, 1.0f);
	}
	else if (dir.y == 0.0f) // horizontal
	{
		v[0] = float4(input[0].posL.x, input[0].posL.y - r, 0.0f, 1.0f);
		v[1] = float4(input[0].posL.x, input[0].posL.y + r, 0.0f, 1.0f);
		v[2] = float4(input[1].posL.x, input[1].posL.y - r, 0.0f, 1.0f);
		v[3] = float4(input[1].posL.x, input[1].posL.y + r, 0.0f, 1.0f);
	}
	else // Just for now, diagonal lines will be slanted on the ends
	{
		v[0] = float4(input[0].posL.x, input[0].posL.y - r, 0.0f, 1.0f);
		v[1] = float4(input[0].posL.x, input[0].posL.y + r, 0.0f, 1.0f);
		v[2] = float4(input[1].posL.x, input[1].posL.y - r, 0.0f, 1.0f);
		v[3] = float4(input[1].posL.x, input[1].posL.y + r, 0.0f, 1.0f);
	}
	
	// Transform to world space and output as a triangle strip
	GS_OUT output;
	[unroll]
	for (int i = 0; i < 4; ++i)
	{
		output.posH       = mul( v[i], gWVP );
		output.color      = color;
	
		triStream.Append(output);
	}
}

float4 PS(GS_OUT input) : SV_Target
{
	// Return default color
	return input.color;
}


technique10 LineTech
{
	pass P0
	{
		SetVertexShader( CompileShader( vs_4_0, VS() ) );
        SetGeometryShader( CompileShader ( gs_4_0, GS() ) );
        SetPixelShader( CompileShader( ps_4_0, PS() ) );
	}
}
