/**
  * Material.fx
  *
  * Author:        Peter B. Tummillo
  * Last Modified: 3/25/12
  *
  * Description:   Renders an object with a color determined by its material properties.
  */

/////////////////////////////////////////////////////////////////////
// Constant Buffers
/////////////////////////////////////////////////////////////////////

cbuffer cbPerFrame
{
	float4x4 gVP;

	//float3 gCameraPosition;

	//float4 gLightColor    = float3( 1.0f, 1.0f, 1.0f, 1.0f ); // Light color
	//float3 gLightPosition = float3( 50.0f, 10.0f, 0.0f );     // Light position
};

cbuffer cbPerObject
{
	float4x4 gWorld;
	
	//float4 gAmbient  = float4( 0.2f, 0.2f, 0.2f );
	float4 gDiffuse  = float4( 0.8f, 0.8f, 0.8f, 1.0f );
	//float4 gSpecular = float4( 1.0f, 1.0f, 1.0f );

	//float gAlpha     = 1.0f;
	//int   gShininess = 32;
};


/////////////////////////////////////////////////////////////////////
// Global Vars
/////////////////////////////////////////////////////////////////////

Texture2D gDiffuseMap;


/////////////////////////////////////////////////////////////////////
// State
/////////////////////////////////////////////////////////////////////

SamplerState gDefaultSampler
{
    Filter   = MIN_MAG_MIP_LINEAR;
    AddressU = CLAMP;
    AddressV = CLAMP;
};


/////////////////////////////////////////////////////////////////////
// Shader Input Structures
/////////////////////////////////////////////////////////////////////

struct VS_INPUT
{
    float3 posL    : POSITION;
    float3 normalL : NORMAL;
    float2 texC    : TEXCOORD;
};

struct PS_INPUT
{
    float4 posH  : SV_POSITION;
    float4 color : COLOR0;
    float2 texC  : TEXCOORD;
};


/////////////////////////////////////////////////////////////////////
// Shaders
/////////////////////////////////////////////////////////////////////

PS_INPUT VS( VS_INPUT input, uniform bool bSpecular )
{
    PS_INPUT output;
    
    // Transform vertex position into world space for lighting,
	// and homogenous clip space for display
	float4x4 WVP = mul( gWorld, gVP);
    //float4 posW = mul( float4(input.posL, 1.0f), gWorld );
    output.posH = mul( float4(input.posL, 1.0f), WVP );

    // Transform the normal into world space for lighting
    //float3 normalW = mul( input.normalL, (float3x3)gWorld );

    // Pass the texture coordinate
	output.color = gDiffuse;
    output.texC = input.texC;

    // Compute the light vector
    //float3 lightVector = normalize( gLightPosition - posW.xyz );

    // Compute the ambient and diffuse components of illumination
    //output.color.rgb = gLightColor * gAmbient;
    //output.color.rgb += gLightColor * gDiffuse * saturate( dot( lightVector, normalW ) );

    // If enabled, calculate the specular term
    /*if ( bSpecular )
    {
        float3 viewVector = normalize( posW.xyz - gCameraPosition );
        float3 reflectionVector = reflect( lightVector, normalW );
        float  phoneValue = saturate( dot( reflectionVector, viewVector ) );

        output.color.rgb += gSpecular * pow( phoneValue, gShininess );
    }*/

    //output.color.a = gAlpha;

    return output;
}

float4 PS( PS_INPUT input, uniform bool bTexture ) : SV_Target
{
    float4 output = input.color;

    // Sample and modulate the texture
    if ( bTexture )
    {
        output.rgb *= gDiffuseMap.Sample( gDefaultSampler, input.texC );
    }

    return output;
}


/////////////////////////////////////////////////////////////////////
// Techniques
/////////////////////////////////////////////////////////////////////

technique10 NoSpecular
{
    pass P0
    {
        SetVertexShader( CompileShader( vs_4_0, VS(false) ) );
        SetGeometryShader( NULL );
        SetPixelShader( CompileShader( ps_4_0, PS(false) ) );
    }
}

technique10 Specular
{
    pass P0
    {
        SetVertexShader( CompileShader( vs_4_0, VS(true) ) );
        SetGeometryShader( NULL );
        SetPixelShader( CompileShader( ps_4_0, PS(false) ) );
    }
}

technique10 TexturedNoSpecular
{
    pass P0
    {
        SetVertexShader( CompileShader( vs_4_0, VS(false) ) );
        SetGeometryShader( NULL );
        SetPixelShader( CompileShader( ps_4_0, PS(true) ) );
    }
}

technique10 TexturedSpecular
{
    pass P0
    {
        SetVertexShader( CompileShader( vs_4_0, VS(true) ) );
        SetGeometryShader( NULL );
        SetPixelShader( CompileShader( ps_4_0, PS(true) ) );
    }
}
