/**
  * Particles.fx
  *
  * Author:        Phil Moccio
  * Last Modified: 3/27/12
  *
  * Description:   ?
  */

Texture2D txDiffuse;
float2 uvScale = float2(1, 1);
float2 uvOffset = float2(0, 0);
float2 dir = float2(1.0f, 0.0f); //right
SamplerState samLinear
{
    Filter = MIN_MAG_MIP_LINEAR;
    AddressU = Wrap;
    AddressV = Wrap;
};

cbuffer cbPerFrame
{
	float4x4 gVP;
};

cbuffer cbPerObject
{
	float4x4 gWorld;
	float t;
	bool rainbowVomit;
};


struct VS_IN
{
	float3 posL   : POSITION;
	float4 color  : COLOR;
};

struct VS_OUT
{
	float4 posH  : SV_POSITION;
	float4 color : COLOR;
	float2 tex : TEXCOORD;
};

struct PS_IN
{
    float4 pos : SV_POSITION;
    float4 color : COLOR;
	float2 tex : TEXCOORD;
};


VS_OUT VS(VS_IN input)
{
	VS_OUT output;

	// Transform vertex position to homogeneous clip space
	float4x4 WVP = mul( gWorld, gVP);
	output.posH = mul( float4(input.posL, 1.0f), WVP );
	
	// Just pass vertex color into the pixel shader
    output.color = input.color;
	
	//UV coordinates of the input vertex are unimportant
	output.tex = float2(0.0, 0.0);

	return output;
}


[maxvertexcount(6)]
void GS( point VS_OUT input[1], inout TriangleStream<VS_OUT> triStream )
{
	PS_IN box[4];
	
	float4x4 topLeft = {1.0, 0.0, 0.0, -0.01,
						0.0, 1.0, 0.0, 0.015,
						0.0, 0.0, 1.0, 0.0,
						0.0, 0.0, 0.0, 1.0};

	float4x4 topRight = {1.0, 0.0, 0.0, 0.01,
						 0.0, 1.0, 0.0, 0.015,
						 0.0, 0.0, 1.0, 0.0,
						 0.0, 0.0, 0.0, 1.0};

	float4x4 bottomLeft = {1.0, 0.0, 0.0, -0.01,
						   0.0, 1.0, 0.0, -0.015,
						   0.0, 0.0, 1.0, 0.0,
						   0.0, 0.0, 0.0, 1.0};

	float4x4 bottomRight = {1.0, 0.0, 0.0, 0.01,
							0.0, 1.0, 0.0, -0.015,
						    0.0, 0.0, 1.0, 0.0,
						    0.0, 0.0, 0.0, 1.0};


	box[0].pos = mul(topLeft, input[0].posH);
	box[0].color = input[0].color;
	box[0].tex = float2(0.0, 0.0);

	box[1].pos = mul(topRight, input[0].posH);
	box[1].color = input[0].color;
	box[1].tex = float2(1.0, 0.0);

	box[2].pos = mul(bottomLeft, input[0].posH);
	box[2].color = input[0].color;
	box[2].tex = float2(0.0, 1.0);

	box[3].pos = mul(bottomRight, input[0].posH);
	box[3].color = input[0].color;
	box[3].tex = float2(1.0, 1.0);
	

	triStream.Append(box[0]);
	triStream.Append(box[1]);
	triStream.Append(box[2]);
	triStream.RestartStrip();
	triStream.Append(box[1]);
	triStream.Append(box[3]);
	triStream.Append(box[2]);
	triStream.RestartStrip();
}

float4 PS(VS_OUT input) : SV_Target
{
	//Color flash
	float4 c;
	
	if(rainbowVomit)
		c = float4(0.5*cos(t*15) + 0.5, 0.5*sin(t*5)  + 0.5, 0.5*sin(t*15) + 0.5, 1.0);
	else
		c = float4(1.0,1.0,1.0,1.0);


	return txDiffuse.Sample( samLinear, input.tex ) * c;
}


technique10 ParticleTech
{
	pass P0
	{
		SetVertexShader( CompileShader( vs_4_0, VS() ) );
        SetGeometryShader(CompileShader(gs_4_0, GS()));
        SetPixelShader( CompileShader( ps_4_0, PS() ) );
	}
}
