/**
  * Texture.fx
  *
  * Author:        Peter B. Tummillo
  * Last Modified: 1/19/12
  *
  * Description:   Renders an object with a texture.
  */

/////////////////////////////////////////////////////////////////////
// Constant Buffers
/////////////////////////////////////////////////////////////////////

cbuffer cbPerFrame
{
	float4x4 gVP;
};

cbuffer cbPerObject
{
	float4x4 gWorld;
};


/////////////////////////////////////////////////////////////////////
// Global Vars
/////////////////////////////////////////////////////////////////////

Texture2D gDiffuseMap;


/////////////////////////////////////////////////////////////////////
// State
/////////////////////////////////////////////////////////////////////

SamplerState gDefaultSampler
{
	Filter   = MIN_MAG_MIP_LINEAR;
	AddressU = CLAMP;
	AddressV = CLAMP;
};


/////////////////////////////////////////////////////////////////////
// Shader Input Structures
/////////////////////////////////////////////////////////////////////

struct VS_IN
{
	float3 posL   : POSITION;
	float2 texC   : TEXCOORD;
};

struct VS_OUT
{
	float4 posH  : SV_POSITION;
	float2 texC  : TEXCOORD;
};


/////////////////////////////////////////////////////////////////////
// Shaders
/////////////////////////////////////////////////////////////////////

VS_OUT VS(VS_IN input)
{
	VS_OUT output;

	// Transform vertex position to homogeneous clip space
	float4x4 WVP = mul( gWorld, gVP );
	output.posH = mul( float4( input.posL, 1.0f ), WVP );
	
	// Just pass vertex texture coordinate into the pixel shader
	output.texC  = input.texC;
	
	return output;
}

float4 PS(VS_OUT input) : SV_Target
{
	// Sample texture
	float4 output = gDiffuseMap.Sample( gDefaultSampler, input.texC );
	return output;
}


/////////////////////////////////////////////////////////////////////
// Techniques
/////////////////////////////////////////////////////////////////////

technique10 Texture
{
	pass P0
	{
		SetVertexShader( CompileShader( vs_4_0, VS() ) );
        SetGeometryShader( NULL );
        SetPixelShader( CompileShader( ps_4_0, PS() ) );
	}
}
