/**
  * TextureInstanced.fx
  *
  * Author:         Peter B. Tummillo
  * Last Modified:  2/27/12
  * 
  * Description:    Renders an instanced object with a texture.
  */

cbuffer cbPerFrame
{
	float4x4 gWVP; 
};


Texture2D gDiffuseMap;


SamplerState g2DSampler
{
	Filter   = MIN_MAG_MIP_LINEAR;
	AddressU = CLAMP;
	AddressV = CLAMP;
};


struct VS_IN
{
	float3 posL                  : POSITION;
	float2 texC                  : TEXCOORD;
	float4 color                 : COLOR;
	row_major float4x4 transform : TRANSFORM;
};

struct VS_OUT
{
	float4 posH   : SV_POSITION;
	float2 texC   : TEXCOORD;
	float4 color  : COLOR;
};


VS_OUT VS(VS_IN input)
{
	VS_OUT output;

	// Determine position of this instance
	float4 instancePos = mul( float4(input.posL, 1.0f), input.transform );

	// Transform vertex position to homogeneous clip space
	output.posH = mul( instancePos, gWVP );
	
	// Just pass color and texture coordinate into the pixel shader
	output.texC  = input.texC;
	output.color = input.color;

	return output;
}

float4 PS(VS_OUT input) : SV_Target
{
	// Sample the texture
	float4 diffuse = gDiffuseMap.Sample( g2DSampler, input.texC );

	// Mix texture with color
	return input.color * diffuse;
}


technique10 TextureInstancedTech
{
	pass P0
	{
		SetVertexShader( CompileShader( vs_4_0, VS() ) );
        SetGeometryShader( NULL );
        SetPixelShader( CompileShader( ps_4_0, PS() ) );
	}
}
